package sound;

import javax.sound.sampled.*;
import utilities.WavRoutines;
import java.io.*;
import java.net.URL;

/**
 * class to encapsulate information about an audio sample 
 */
public class AudioSample {

    AudioInputStream   inputStream;
    public AudioFormat format;
    byte[]             audioBytes;
    static final boolean DEBUG = false;
 
    /**
     * length of cutoff signal in samples
     **/
    int                signalCutoff = 0; 
    /**
     * duration in seconds
     **/
    double             duration; 

    int[]              displayData = null;
    int                windowSize;

    /** 
     * frame increment in samples? 
     **/
    int                winIncrement;

    private  File      file;
    private  URL       url;
   /**
     * constructor
     * @param file filename of audiosample
     */
    public AudioSample(URL url) 
	throws Exception 
    {
	inputStream = AudioSystem.getAudioInputStream(url);
	format      = inputStream.getFormat();

	signalCutoff   = 
	    WavRoutines.maxLengthInSamples(inputStream,
					   format);
	if (DEBUG) {
	    System.out.println("starting signal cutoff at: " + signalCutoff);
	}
	inputStream = AudioSystem.getAudioInputStream(url);
	this.url = url;

    }
 

    /**
     * constructor
     * @param file filename of audiosample
     */
    public AudioSample(File file) 
	throws Exception 
    {
	this.file = file;
	inputStream = AudioSystem.getAudioInputStream(file);
	format      = inputStream.getFormat();

	signalCutoff   = 
	    WavRoutines.maxLengthInSamples(inputStream,
					   format);
	if (DEBUG) {
	    System.out.println("starting signal cutoff at: " + signalCutoff);
	}
	inputStream = AudioSystem.getAudioInputStream(file);
    }
    /**
     * returns the signal size, in samples
     */
    public int getSignalSizeInSamples() {
	return signalCutoff;
    }

    /** 
     * returns a new copy of the input audio stream 
     */
    public AudioInputStream getAudioInputStream() {
    // would this be any more efficient if I made a copy 
	// of the original instead? 
	try {
	    if (file != null) {
		return AudioSystem.getAudioInputStream(file);
	    }
	    else if (url != null) {
		return AudioSystem.getAudioInputStream(url);
	    }
	}
	catch (Exception ex) { 
	    ex.printStackTrace();
	    return null;
	}
	return null;
    }
    /**
     * get file name of audio sample 
     */
    public String getFileName() {
	if (file != null) {
	    return file.getAbsolutePath();
	}
	else if (url != null) {
	    return url.getFile();
	}
	else return null;
    }
    /**
     * given windowsize, cuts off signal to nearest whole window. 
     */
    public void setSignalCutoff(int windowSize) {
	signalCutoff = (signalCutoff/windowSize)*windowSize;
    }

    /**
     * set window increment (non-overlap part)
     * @param winInc number of samples in window increment
     */
   public void setWindowIncrement(int winInc) {
	winIncrement = winInc;
    }

    /**
     * set total window size
     * @param winSize number of samples in window 
     */
    public void setWindowSize(int winSize) {
	windowSize = winSize;
    }

    /**
     * get frame increment (number of audioformat.frames in winIncrement)     
     */
    public int getFrameIncrement() {
	return  (winIncrement*format.getFrameSize());
    }
    
    /**
     *  returns a format of the samples ready to be graphically displayed
     */
    public int [] getDisplayFormat() {
	if (displayData != null) {
	    return displayData;
	}
	else {
	    if (audioBytes == null) {
		makeAudioBytes();
	    }
	    convertToDisplayFormat();
	    return displayData;
	}
    }

    /**
     * puts the audio sample into accessible byte array form
     */
    private void makeAudioBytes() {

	int numBytes = signalCutoff*getSizeInBytes();
	if (DEBUG) {
	    System.out.println("numBytes: " + numBytes);
	}
	try {
	    audioBytes = new byte[numBytes];
	    int numRead = 0;
	    
	    while (numRead < numBytes) {
		numRead += 
		    inputStream.read(audioBytes, numRead, numBytes-numRead);
		//    System.out.println("AudioSample.makeAudioBytes: numRead: " + numRead);

	    }

	    // is inputStream now null? 
	} catch (Exception ex) { 
	    ex.printStackTrace();
	    return; 
	}
    }

    /**
     * get a reference to the byte array of the audio sample. 
     */
    public byte [] getAudioBytes() {
	return audioBytes;
    }

    /**
     *  get the size of a sample in bytes (usually 1 or 2)
     */
    public int getSizeInBytes() {
	return format.getSampleSizeInBits() / 8;
    }

    /**
     *  get number of bytes that are consumed in 1 second
     */
    public float getBytesPerSecond()
    {
	return format.getFrameRate() * format.getFrameSize();
    }

    /**
     *  convert sample to display format
     */
    private void  convertToDisplayFormat() {
	//  this converts various soundfile formats. 
	// into 1 that can be displayed
	if (DEBUG) {
	    System.out.println("audioBytes size: " + audioBytes.length);
	    System.out.println("signalCutoff: " + signalCutoff);
	}
	displayData = new int[signalCutoff];

	if (format.getSampleSizeInBits() == 16) {
	    // System.out.println("samplesizeinbits = 16");
	    
	    if (format.isBigEndian()) {
		// System.out.println("format big endian");
		
		for (int i = 0; i < signalCutoff; i++) {
		    /* First byte is MSB (high order) */
		    int MSB = (int) audioBytes[2*i];
		    /* Second byte is LSB (low order) */
		    int LSB = (int) audioBytes[2*i+1];
		    displayData[i] = MSB << 8 | (255 & LSB);
		}
	    } else {
		// System.out.println("format small endian");
		
		for (int i = 0; i < signalCutoff; i++) {
		    /* First byte is LSB (low order) */
		    int LSB = (int) audioBytes[2*i];
		    /* Second byte is MSB (high order) */
		    int MSB = (int) audioBytes[2*i+1];
		    displayData[i] = MSB << 8 | (255 & LSB);
		}
	    }
	} else if (format.getSampleSizeInBits() == 8) {
	    // System.out.println("samplesizeinbits = 8");
	    
	    if (format.getEncoding().toString().startsWith("PCM_SIGN")) {
		for (int i = 0; i < signalCutoff; i++) {
		    displayData[i] = audioBytes[i];
		}
	    } else {
		for (int i = 0; i < signalCutoff; i++) {
		    displayData[i] = audioBytes[i] - 128;
		}
	    }
	}	    
    }
}
