package utilities;

/**
 * methods for calculating the information content of an array
 *
 */
public class InfoCost {
    
    /**
     * thresh()
     *
     * Threshold information cost function.
     *
     * Calling sequence and basic algorithm:
     *   thresh( U, LEAST, FINAL, EPSILON ):
     *      Let COST = 0
     *      For K = LEAST to FINAL
     *	  If absval(U[K]) > EPSILON then
     *	    COST += 1
     *      Return COST
     *
     * Inputs:
     *	double []u	   This must point to a preallocated array.
     *
     *	(int)least		These are the least and final valid
     *	(int)final		  indices in `u[]'.
     *
     *	(double)epsilon		This is a positive, absolute threshold.
     *
     * Output:
     *	(double)thresh		The return value is the number of elements
     *				  of `u[]' whose absolute values are
     *				  greater than `epsilon'.
     *
     *  Assumptions:
     *	1. epsilon>=0
     */
    
public static int thresh(			// Threshold cost function.
    double [] u,	/* Pdoublelocated array of coefficients. */
    int least,	/* Least valid `u[]' index.            */
    int fin,	        /* Greatest valid `u[]' index.         */
    double epsilon)	/* Absolute value threshold.           */
    {
	int cost;
	int k;
	
	if (epsilon < 0) {
	    System.out.println("epsilon should be > 0");
	    return 0;
	}

	cost = 0;
	for( k=least; k<=fin; k++ ) {
	    if( Math.abs(u[k])>epsilon ) {
		cost ++;
	    }
	}
	return(cost);	
    }
    
    
    /**
     * l1norm()
     *
     * Absolute summation information cost function.
     *
     * Calling sequence and basic algorithm:
     *   l1norm( U, LEAST, FINAL ):
     *      Let COST = 0
     *      For K = LEAST to FINAL
     *	   COST += absval( U[K] )
     *      Return COST
     *
     * Inputs:
     *	(const double *)u      	This must point to a preallocated array.
     *
     *	(int)least		These are the least and final valid
     *	(int)final		  indices in `u[]'.
     *
     * Output:
     *	(double)l1norm		The return value is the sum of the absolute
     *				  values of the elements of `u[]'.
     */
    
    public static double l1norm(// Absolute summation cost function.   
		  double [] u,  // Preallocated array of coefficients.
		  int least,	// Least valid `u[]' index.            
		  int fin)	// Greatest valid `u[]' index.         
    {
	double cost;
	int k;

	cost = 0;
	for( k=least; k<=fin; k++ ) {
	    cost += Math.abs(u[k]);
	}
	return(cost);
    }


    /**
     * lpnormp()
     *
     * p-th power summation information cost function.
     *
     * Calling sequence and basic algorithm:
     *   lpnormp( U, LEAST, FINAL, P ):
     *      Let COST = 0
     *      For K = LEAST to FINAL
     *         Let ABSU = absval( U[K] )
     *         If ABSU > 0 then
     *           COST += exp( P*log( ABSU ) )
     *      Return COST
     *
     * Inputs:
     *	(const double) u	This must point to a preallocated array.
     *
     *	(int)least		These are the least and final valid
     *	(int)fin		  indices in `u[]'.
     *
     *	(double)p		Use this power in the summation.
     *
     * Output:
     *	(double)lpnormp		The return value is the sum of the `p'-th
     *				 power of the absolute values of the
     *				 elements of `u[]'.
     *
     */
    public static double lpnormp(	// p-th power summation cost function. 
		   double [] u,	// Preallocated array of coefficients. 
		   int least,	// Least valid `u[]' index.            
		   int fin,	// Greatest valid `u[]' index.         
		   double p)	// Power to use in summation.        
    {
	double cost, absu;
	int k;

	cost = 0;
	for( k=least; k<=fin; k++ )
	    {
		absu = Math.abs(u[k]);
		if( absu>0 ) {
		    cost += Math.exp( p * Math.log(absu) );
		}
	    }
	return(cost);
    }

/**
 * ml2logl2()
 *
 * ``Entropy'' information cost function.
 *
 * Calling sequence and basic algorithm:
 *   ml2logl2( U, LEAST, FINAL ):
 *      Let COST = 0
 *      For K = LEAST to FINAL
 *         Let USQ =  U[K] * U[K]
 *         If USQ > 0 then
 *            COST -= USQ * log( USQ )
 *      Return COST
 *
 * Inputs:
 *	(double [])u	    This must point to a preallocated array.
 *
 *	(int)least		These are the least and final valid
 *	(int)final		  indices in `u[]'.
 *
 * Output:
 *	(double)ml2logl2	 The return value is the sum of `u*u*log(u*u)'
 *				 for all nonzero values of `u*u'.
 */
public static double ml2logl2(	   // ``Entropy'' cost function.    
		    double [] u,   // Preallocated array of coefficients. 
		    int least,	   // Least valid `u[]' index.            
		    int fin)	   // Greatest valid `u[]' index.         
    {
	double cost, usq;
	int k;

	cost = 0;
	for( k=least; k<=fin; k++ )
	    {
		usq = (u[k]*u[k]);
		if( usq>0 ) {
		    cost -= usq * Math.log(usq);
		}
	    }
	return(cost);
    }

/**
 * logl2()
 *
 * Gauss-Markov information cost function.
 *
 * Calling sequence and basic algorithm:
 *   logl2( U, LEAST, FINAL ):
 *      Let COST = 0
 *      For K = LEAST to FINAL
 *         COST += log( U[K]*U[K] )
 *      Return COST
 *
 * Inputs:
 *	( double [])u		This must point to a preallocated array.
 *
 *	(int)least		These are the least and final valid
 *	(int)final		  indices in `u[]'.
 *
 * Output:
 *	(double)logl2		The return value is the sum of `log(u*u)'
 *				 for all values of `u*u'.
 */
   public  static double logl2(	/* Gauss-Markov cost function.         */
		 double [] u,	/* Preallocated array of coefficients. */
		 int least,	/* Least valid `u[]' index.            */
		 int fin)	/* Greatest valid `u[]' index.         */
    {
	double cost;
	int k;

	cost = 0;
	for( k=least; k<=fin; k++ ) {
	    cost += Math.log(u[k]*u[k]);
	}
	return(cost);
    }
    /**
     * tdim()
     *
     * Theoretical dimension of a sequence
     *
     * Calling sequence and basic algorithm:
     *   tdim( U, LEAST, FINAL ):
     *      Let MU2LOGU2 = 0
     *      Let ENERGY = 0
     *      For K = LEAST to FINAL
     *         Let USQ =  U[K] * U[K]
     *         If USQ > 0 then
     *            MU2LOGU2 -= USQ * log( USQ )
     *            ENERGY += USQ
     *      If ENERGY > 0 then
     *         Let THEODIM =  ENERGY * exp( MU2LOGU2 / ENERGY )
     *      Else
     *         Let THEODIM = 1
     *      Return THEODIM
     *
     * Inputs:
     *	(double []) u		This must point to a preallocated array.
     *
     *	(int)least		These are the least and final valid
     *	(int)final		  indices in `u[]'.
     *
     * Output:
     *	(double)tdim		The return value is exp of th sum of
     *				 `u*u*log(u*u)' for all nonzero values
     *				 of `u*u'.
     */
    public static double tdim(	 // Theoretical dimension.         
		double [] u,	 // Preallocated array of coefficients.
		int least,	 // Least valid `u[]' index.            
		int fin)	 // Greatest valid `u[]' index.         
    {
	double theodim, mu2logu2, energy, usq;
	int k;

	mu2logu2 = 0;
	energy = 0;
	for( k=least; k<=fin; k++ )
	    {
		usq =  u[k] * u[k];
		if( usq>0 )
		    {
			mu2logu2 -= usq * Math.log( usq );
			energy += usq;
		    }
	    }
	if(energy>0) {
	    theodim =  energy * Math.exp( mu2logu2 / energy );
	}
	else {
	    theodim = 0.0;
	}
	return(theodim);
    }
}
