package wavelet;

import	java.io.File;
import	java.io.IOException;
import  cern.colt.Arrays;
import	javax.sound.sampled.AudioInputStream;
import	javax.sound.sampled.AudioFormat;
import  utilities.*;

/**
 * Main Wavelet Transform Class 
 * Description: Performs the wavelet transform on array of doubles 
 * @version $Date: 2001/02/11 $
 * @author  Reynald Hoskinson
 * @since   JDK 1.2.2
*/

public class Wavelet
{
    static final int MIN_ORDER = 2;
    
    // data structures to hold levels of wavelet transform
    
    /** I store the coefficients in a 2-Dim tree
     * The first dim represents the depth.  
     * The second is the smoothcoefs followed by detcoefs
     **/
    public  double[][] wavTree;
    private double[] inDataArray;
    public  int depthT;
    public  int inSignalCutoff;
    public  AudioFormat audioFormat;

    /**
     * Constructs an Wavelet object
     * @param intputSignal The one-dimensional input signal
     */
    public Wavelet(double[] inputSig) {
	inDataArray = inputSig;
	initialize(inDataArray.length);
    }

    /**       
     * Constructs an Wavelet object with length different than array size. 
     * @param intputSignal The one-dimensional input signal
     * @param intputLength Length of signal to analyze (<= inputSignal.length) 
    */
    public Wavelet(double[] inputSig, int inputLength) {
	inDataArray = inputSig;
	initialize(inputLength);
    }

    /**       
     * Constructs an Wavelet object using AudioInputStream
     * @param audioIn inputSignal to analyze
    */
    public Wavelet(AudioInputStream audioIn) {
	audioFormat = audioIn.getFormat();
	// converts sample in from bytes to doubles. 	
	inDataArray = getDoubleData(audioIn);
	// allocate enough memory for a full transform
	initialize(inDataArray.length);
    }

    /**       
     * Constructs an Wavelet object using AudioInputStream
     * @param audioIn inputSignal to analyze
     * @param length Length of signal to analyze (<= audioIn length) 
    */
    public Wavelet(AudioInputStream audioIn, int length) {
	audioFormat = audioIn.getFormat();
	// converts sample in from bytes to doubles. 	
	inDataArray = getDoubleData(audioIn);
	// allocate enough memory for a full transform
	initialize(length);
    }


    /**       
     * Get a new copy of the input array. 
     * @return array copy of input signal
    */
    public double[] getDoubleData() {
	double[] outp = new double[inDataArray.length];
	System.arraycopy(inDataArray, 0, outp, 0, inDataArray.length);	
	return outp;
    }

    /**       
     * Get a new copy of the input array. 
     * @return array copy of input signal
    */
    public double[] getDoubleData(AudioInputStream audioIn) {
        
        double[] dData = WavRoutines.getPowerOfTwoDoubleData(audioIn, 
							     audioFormat);
	depthT = (int)(Math.log(dData.length)/Math.log(2));	
	
        return(dData);
    }	

    /**       
     * Utility routine: conversion routine from double to bytes
     * @param dat (double array to convert (for instance, the inverse wavelet inverse transform)
     * @param byteArray input double array converted to bytes. 
    */
    public void getByteData(double[] dat, byte[] byteArray) {
	/* support only 16-bit first */

	/* find out the length in bytes of the new sample */
	/* assume it has the same number of bits, etc. */

	int sizeInBytes = audioFormat.getSampleSizeInBits() / 8;
	int nSamples    = dat.length * sizeInBytes;
	short tempShort;

	// if (byteArray.length < nSamples)
	// return some sort of exception
	
	for (int s = 0; s < dat.length; s++) {
		tempShort = (short)(dat[s] * -Short.MIN_VALUE);
		byteArray[s*2] = (byte) (tempShort & 0xff);	       
		byteArray[s*2+1] = (byte) ((tempShort >> 8) & 0xff);
	}
    }

    /**       
     * Utility routine: conversion routine from double to bytes
     * @param dat (double array to convert (for instance, the inverse wavelet inverse transform)
     * @return input double array converted to new array of bytes. 
    */
    public byte[] getByteData(double[] dat) {
	/* support only 16-bit first */

	/* find out the length in bytes of the new sample */
	/* assume it has the same number of bits, etc. */

	int sizeInBytes = audioFormat.getSampleSizeInBits() / 8;
	int nSamples    = dat.length * sizeInBytes;
	short tempShort;
	
	byte[] newByteArray = new byte[nSamples];
	
	for (int s = 0; s < dat.length; s++) {
		tempShort = (short)(dat[s] * -Short.MIN_VALUE);
		newByteArray[s*2] = (byte) (tempShort & 0xff);	       
		newByteArray[s*2+1] = (byte) ((tempShort >> 8) & 0xff);
	}
	return newByteArray;	
    }
      
    /**       
     * Performs the wavelet transform, and returns an array 
     * with all of the detail coef's, starting from the root
     * @param  wfilter  wavelet filter to use. 
     * @return input double array converted to new array of bytes. 
    */
    public double[] mallatTree(WaveletFilter wfilter)
    {
	// calculate length of mallat Tree array
	// 2^depthT-1 + .. .. 1
	int i,j, ct, inc;
	int treeSize = 0;
		
	for (i = 1; i<= depthT; i++) {
	    treeSize += Math.pow(2, depthT-i);
	}
	
	forwardTransform(depthT, 0, wfilter);	
	double[] malTree = new double[treeSize];

	ct = 1;
	inc = 0;
	int numDetCoefs;
	for (i = depthT-1; i >= 0; i--)
	{
		numDetCoefs = wavTree[i].length/2;
		for(j = numDetCoefs; j < numDetCoefs + ct; j++)
		{
			malTree[inc] = wavTree[i][j];
			inc++;
		}
		ct *=2;
	} 
	return malTree;
    }
   
    /**       
     * performs an inverse wavelet transform of a mallat tree.
     * @param  wfilter  wavelet filter to use 
     * @param  mtree mallat tree 
     * @return reconstitued signal
    */
    public double[] inverseMallatTree(WaveletFilter wFilter, double[] mtree)
    {
	int i,j;
	int lngth;
	int ct = 1;
	int powIndex;
	
	// allocate enough space for full transform
	double[] inversed = new double[inSignalCutoff];

	inversed[0] = wavTree[depthT-1][0];
	inversed[1] = mtree[0];

	for (i=1; i< depthT; i++) {
		// add detail coef from mallat tree 
		powIndex = (int)Math.pow(2,i);				
		// src, src index, dest, dest index, length
		System.arraycopy(mtree, ct, inversed, powIndex, powIndex);
		fa1d(inversed, 0, powIndex, false, wFilter, inversed, 0);
		ct += powIndex;
	}
	fa1d(inversed, 0, inversed.length, false, wFilter, inversed, 0);
	// System.out.println("inversed: ");
	// System.out.println(Arrays.toString(inversed));	
	return inversed;
    }
    
    
    /**       
     * initialization stuff
     * @param inputLength length of portion of the input signal to use
     */
    private void initialize(int inputLength)
    {
	/* ensure a power of two */
	depthT = (int)Math.floor((Math.log(inputLength)/Math.log(2)));

	// System.out.println("depthT: " + depthT);
	inSignalCutoff = (int)Math.pow(2,depthT);
	// System.out.println("insignalcutoff: " + inSignalCutoff);
	// allocate enough memory for a full transform
        wavTree = new double[depthT+1][];
    }
  
    /**
      perform a n level wavelet transform on the data in inDataArray
      @param levels number of levels of the wavelet transform to do. 
      @param inSignalStart place to start in the input signal 
      @param wFilter wavelet filter to use.  
    */
    //  note: used in mallatTree     
    public void forwardTransform(int levels, int inSignalStart, 
				 WaveletFilter wFilter)
    {
	wavTree[0] = new double[inSignalCutoff];
	fa1d(inDataArray, inSignalStart, inSignalCutoff, true, wFilter, 
	     wavTree[0], 0);

	//System.out.println("After first level: ");	  
	//printTree(0);
	
	for (int i=1; i< levels; i++) {	    	    
	    wavTree[i] = new double[(wavTree[i-1].length)/2];
	    // System.out.println("Wavelet - levelArray: " + wavTree[i].length);
	    
	    fa1d(wavTree[i-1], 0, wavTree[i].length, true, wFilter, wavTree[i], 0);
	}
    }	    

    /**
     * perform a 1 level inverse wavelet transform on the data in inDataArray
     * @param level level of wavelet transform to do
     * @param wFilter filter to use
    */    
    private double[] inverseTransform(int level, WaveletFilter wFilter)
    {
	double [] outArray = new double[wavTree[level].length];
	fa1d(wavTree[level], 0, wavTree[level].length, false, wFilter, outArray, 0);
	return outArray;
    }	    
    
    /** 
     * debug utility: prints array to standard out 
     * @param level level of wavelet transform to print out.  
     */
    public void printTree(int level)
    {
      System.out.println("Wavelet - printtree: " + level); 
      for (int i = 0; i < wavTree[level].length ; i++)
	{
	  System.out.println(wavTree[level][i]);
	}	       
    }
    
    /**
     * returns a copy of one level's wavelet (smooth)  coefficients
     * @param level specify level wavelet transform to return
     * @return copy of specified level's average coefficients
     */    
    public double[] smoothCoefs(int level) {
	double[] outArray = new double[wavTree[level].length/2];	
	System.arraycopy(wavTree[level], 0, outArray, 0, outArray.length);
	return outArray;
    }
    
    /**
     * returns a copy of one level's detail coefficients
     * @param level specify level wavelet transform to return
     * @return copy of specified level's average coefficients
     */    
    public double[] detCoefs(int level) {
	double[] outArray = new double[wavTree[level].length/2];
	System.arraycopy(wavTree[level], 
			 (outArray.length)-1, outArray, 0, outArray.length);
	return outArray;
    }

    // a bit of a hack. I should make infoCost a Class that I can 
    // pass in as a paramter, that will take an array and relavent 
    // indices and produce a double.  any class that has such a 
    // function will do.  i would have a standard function name, like
    // computeCost. -> i could make an interface. 

    
    /**
     * computes the l1 norm of a level of detail coefficients
     * @param level specify level wavelet transform to use
     * @return l1 norm of a level of detail cofficients
     */    
    public double detCoefCost(int level) {
	int detLength = wavTree[level].length/2;
	 return InfoCost.l1norm(wavTree[level], detLength-1, detLength);
	//return InfoCost.ml2logl2(wavTree[level], detLength-1, detLength);
    }

    /**
     * computes the l1 norm of a level of avg coefficients
     * @param level specify level wavelet transform to use
     * @return l1 norm of a level of avg cofficients
     */    
    public double smoothCoefCost(int level) {
	int smoothLength = wavTree[level].length/2;
	return InfoCost.l1norm(wavTree[level], 0, smoothLength);
    }

    /**
     * computes the avg amplitude of this segment
     * @return median amplitude of this window. 
     */    
    public double avgAmplitude() {
	double avg = 0;
	for (int i=0; i < wavTree[0].length; i++) {
	    avg += Math.abs(wavTree[0][i]);
	}
	avg /= (double)(wavTree[0].length);
	// System.out.println("Window avg: " + avg);
	return avg;
    }
			    
  /* wxfrm_1d_varstep -- 1-dimensional discrete wavelet transform
     with variable step size */

    /**
     * intermediate routine to compute discrete wavelet wavelet transform 
     * @param aIn original array
     * @param inStart  index to start on in aIn
     * @param incA spacing of elements in input/output arrays
     * @param nA size of portion of input array to use
     * @param isFwd TRUE <=> forward transform FALSE - inverse
     * @param wfltr wavelet filter to use 
     * @param aXf space for transformed array 
     * @param aXfStart index to start from in transformed array
     */    
    static void oneD_Varstep
	(double aIn[],	    /* in: original array */
	 int aInStart,
	 int incA,	    /* in: spacing of elements in aIn[] and aXf[] */
	 int nA,            /* in: size of aIn (must be power of 2) */
	 boolean isFwd,		/* in: TRUE <=> forward transform */
	 WaveletFilter wfltr,	/* in: wavelet filter to use */
	 double aXf[],
	 int aXfStart)	        /* out: transformed array */
    {
	int iA;
	if (nA < MIN_ORDER) {
	    System.out.println("ERROR: Wavelet.java: na < MIN_ORDER");
	    return;
	}
    
	if (isFwd) {
	    /*
	     *	Start at largest hierarchy, work towards smallest.
	     */
	    WaveletConvolve.convolve(wfltr, isFwd, aIn, aInStart, incA, nA,aXf,
				     aXfStart);
	    for (iA = nA / 2; iA >= MIN_ORDER; iA /= 2)
		WaveletConvolve.convolve(wfltr,isFwd,aXf,aXfStart,incA,iA, aXf,
					 aXfStart);
	} else {
	    /*
	     *	Start at smallest hierarchy, work towards largest.
	     */
	    if ((aXf != aIn) || (aInStart != aXfStart)) {
		for (iA = 0; iA < nA; iA++)
		    aXf[incA * iA + aXfStart] = aIn[incA * iA + aInStart];
		/* required for inverse */
	    }
	    for (iA = MIN_ORDER; iA <= nA; iA *= 2)
		WaveletConvolve.convolve(wfltr, isFwd, aXf, aXfStart, incA,
					 iA, aXf, aXfStart);
	}
	return;
    }
    
    /**
     * intermediate routine to compute discrete wavelet wavelet transform 
     * just calls oneDVarStep with a step size of 1
     * @param a original array
     * @param aStart  index to start on in original array
     * @param nA size of portion of input array to use
     * @param isFwd TRUE <=> forward transform FALSE - inverse
     * @param wfltr wavelet filter to use 
     * @param aXf space for transformed array 
     * @param aXfStart index to start from in transformed array
     */
    public static void fa1d
	(double a[],	      /* in: original array */
	 int aStart,          /* start index in A */ 
	 int nA,              /* in: size of a (must be power of 2) */
	 boolean isFwd,	      /* in: TRUE <=> forward transform */
	 WaveletFilter wfltr, /* in: wavelet filter to use */
	 double aXf[],        /* out: transformed array */
	 int aXfStart)        /* start index */
	 	        
    {
	oneD_Varstep(a, aStart, 1, nA, isFwd, wfltr, aXf, aXfStart);    
	return;
    }

    
    // 
    // returns the mixed signal
    // assume both have the same length. 
    
    /**
     * mixes two mallat trees. 
     * @param other oher mallat tree to mix with (one is implicit)
     * @param wflilter wavelet filter to use
     * @return new mallat tree which is a mix of the first two
     */
    public double[] mix(Wavelet other, WaveletFilter wfilter) 
    {
	// create temporary mallat tree. 
	double[] mtree1 = mallatTree(wfilter);
	double[] mtree2 = other.mallatTree(wfilter);
	
	mixDoub(mtree1,mtree2,mtree1);
	
	 
	// performs an inverse transform of a mallat tree.

	System.out.println("do wavelet inverse transform");
	return inverseMallatTree(wfilter, mtree1);
	
	/*
	  there is no function that just does a 
	  inverse of the whole tree. 
	  
	  I have my choice of 
	  1) inverse of a mallat tree. 
	  2) inverse level by level . 
	  
	  So, I make a mallat tree of each. 
	  then I mix them. 
	  then I inverse that. 

	*/
    }

    /**
     * Mixes two arrays of doubles: just a linear crossfade right now. 
     * @param signalA first signal 
     * @param signalB second signal 
     * @param mixOut space to put the result
     */
    public void mixDoub(double [] signalA,double [] signalB, double [] mixOut) 
    {

	double iter = 1/(double)mixOut.length;
	double line = 0;

	for (int i=0; i < mixOut.length; i++) {
	    mixOut[i] = ((1-line)*(signalA[i])) + (line*(signalB[i]));
	    line += iter;
	}
    }


    /**
     * choose a wavelet filter from an ordered list. 
     * @param n wavelet filter number
     * @return the wavelet filter associated with this number
     */
    public static WaveletFilter chooseFilter(int n) {
	if (n == 0) {
	    return WaveletFilter.wfltrAdelsonSimoncelliHingorani;
	}
	else if (n == 1) {
	    return WaveletFilter.wfltrAntoniniBarlaudMatheiuDaubechies_4_4;
	}
	else if (n == 2) {
	    return WaveletFilter.wfltrBattleLemarie;
	}
	else if (n == 3) {
	    return WaveletFilter.wfltrBurtAdelson;
	}
	else if (n == 4) {
	    return WaveletFilter.wfltrCoiflet_2;
	}
	else if (n == 5) {
	    return WaveletFilter.wfltrCoiflet_4;
	}
	else if (n == 6) {
	    return WaveletFilter.wfltrCoiflet_6;
	}
	else if (n == 7) {
	    return WaveletFilter.wfltrDaubechies_4;
	}
	else if (n == 8) {
	    return WaveletFilter.wfltrDaubechies_6;
	}
	else if (n == 9) {
	    return WaveletFilter.wfltrDaubechies_8;
	}
	else if (n == 10) {
	    return WaveletFilter.wfltrDaubechies_10;
	}
	else if (n == 11) {
	    return WaveletFilter.wfltrDaubechies_12;
	}
	else if (n == 12) {
	    return WaveletFilter.wfltrDaubechies_20;
	}
	else if (n == 13) {
	    return WaveletFilter.wfltrHaar;
	}
	else if (n == 14) {
	    return WaveletFilter.wfltrPseudocoiflet_4_4;
	}
	else if (n == 15) {
	    return WaveletFilter.wfltrSpline_2_2;
	}
	else if (n == 16) {
	    return WaveletFilter.wfltrSpline_2_4;
	}
	else if (n == 17) {
	    return WaveletFilter.wfltrSpline_3_3;
	}
	else if (n == 18) {
	    return WaveletFilter.wfltrSpline_3_5;
	}
	else return WaveletFilter.wfltrSpline_3_7;
    }

    /**
     * miscallaneous old code
     * @param args input args
     */
    public static void Othermain(String[] args)
    {
	double[] in1 = new double[16];
        for (int i = 0; i < 16; i++) {
	    in1[i] = i;
	}
	
	double[] in2 = new double[16];
        for (int i = 0; i < 16; i++) {
	    in2[i] = 15-i;
	}
	
	System.out.println("arrs1: " + Arrays.toString(in1));
	System.out.println("arrs2: " + Arrays.toString(in2));

	Wavelet w1 = new Wavelet(in1);
	Wavelet w2 = new Wavelet(in2);
	Integer wfl = new Integer(args[0]);
	WaveletFilter wfltr = w1.chooseFilter(wfl.intValue());

	System.out.println("do wavelet transform");		
	w1.forwardTransform(w1.depthT, 0, wfltr);
	w2.forwardTransform(w2.depthT, 0, wfltr);

	double[] mtree1 = w1.mallatTree(wfltr);	
	System.out.println("arrsIni: " + 
Arrays.toString(w1.inverseMallatTree(wfltr, mtree1)));
	
	double[] out = w1.mix(w2, wfltr);
	System.out.println("arrsMix: " + Arrays.toString(out));
    }

    
    public static void main(String[] args)
    {

	// load in two sounds, mix their wavelet trees, 
        // output new sound. 

	// load a signal. 
  	// input sample
	java.io.FileInputStream soundfile;
	double[] transArray;
	AudioFormat audioFormat = null;	
	int i;
	int log2n;
	
	if (args.length < 3) {
	    System.out.println("Wavelet: usage:");
	    System.out.println("\tjava Wavelet <soundfile1> <soundfile2> <WaveletFilter");
	    System.exit(1);
	}
	Integer wfl = new Integer(args[2]);

	AudioInputStream  audioInputStream1  = 
	    WavRoutines.getAudioInputStream(args[0]);	
	AudioInputStream  audioInputStream2  = 
	    WavRoutines.getAudioInputStream(args[1]);	
	
	Wavelet w1 = new Wavelet(audioInputStream1);
	Wavelet w2 = new Wavelet(audioInputStream2, w1.inSignalCutoff);
	WaveletFilter wfltr = w1.chooseFilter(wfl.intValue());

	System.out.println("do wavelet transform");		
	w1.forwardTransform(w1.depthT, 0, wfltr);
	w2.forwardTransform(w2.depthT, 0, wfltr);
	
	double[] out = w1.mix(w2, wfltr);

	WavRoutines.fileDoubles(out,w1.audioFormat);
    }    
}
