package wavelet;

import utilities.*;

/**
 * convolution routines for wavelet transform.
 */
public class WaveletConvolve {

  private static double aTmp[];

    /**
     * perform one convolution of a (general) wavelet transform 
     * @param wfltr in: convolving filter 
     * @param isFwd in: TRUE <=> forward transform 
     * @param aIn in: input data 
     * @param aInStart start index in input array
     * @param incA in: spacing of elements in aIn[] and aXf[] 
     * @param n in: size of aIn and aXf 
     * @param aXf  out: output data (OK if == aIn)    
     * @param aXfStart start index in aXf 
     */
  public static void convolve    
  (WaveletFilter wfltr,      /* in: convolving filter */
   boolean isFwd,	       /* in: TRUE <=> forward transform */
   double[] aIn,	       /* in: input data */
   int aInStart, 
   int incA,		       /* in: spacing of elements in aIn[] and aXf[] */
   int n,		       /* in: size of aIn and aXf */
   double[] aXf,               /* out: output data (OK if == aIn) */    
   int aXfStart)	       
  {
	int nDiv2 = n / 2;
	int iA, iHtilde, iGtilde, j, jH, jG, i;
	double sum, aInVal;
	boolean flip;

	aTmp = new double[n];
	
	/*
	 *	According to Daubechies:
	 *
	 *	H is the analyzing smoothing filter.
	 *	G is the analyzing detail filter.
	 *	Htilde is the reconstruction smoothing filter.
	 *	Gtilde is the reconstruction detail filter.
	 */

	/* the reconstruction detail filter is the mirror of the analysis smoothing filter */
	int nGtilde = wfltr.nH;
	
	/* the analysis detail filter is the mirror of the reconstruction smoothing filter */
	int nG = wfltr.nHtilde;
	
	if (isFwd) {
	    /*
	     *	A single step of the analysis is summarized by:
	     *		aTmp[0..n/2-1] = H * a (smooth component)
	     *		aTmp[n/2..n-1] = G * a (detail component)
	     */
	    for (i = 0; i < nDiv2; i++) {
		/*
		 *	aTmp[0..nDiv2-1] contains the smooth components.
		 */
		sum = 0.0;
		for (jH = 0; jH < wfltr.nH; jH++) {
		/*
		 *  Each row of H is offset by 2 from the previous one.
	         *
	         *  We assume our data is periodic, so we wrap the aIn[] values
	         *  if necessary.  If we have more samples than we have H
	         *  coefficients, we also wrap the H coefficients.
		 */
		    iA = Utils.wrapMod(2 * i + jH - wfltr.offH, n);

		    /*
		    System.out.println("IA: " + iA);
		    System.out.println("jH: " + jH);
		    System.out.println("length " + aIn.length);

		    System.out.println("aIn[icA*iA + aInStart]: : " + incA*iA + aInStart + aIn[incA * iA + aInStart]);
		    */

		    sum += wfltr.cH[jH] * aIn[incA * iA + aInStart];
		}
		aTmp[i] = sum;
	    
		/*
		 *	aTmp[nDiv2..n-1] contains the detail components.
		 */
	    sum = 0.0;
	        /*
	         *	imsG <= 0 <=> initial flip
		 */
	    flip = (wfltr.imsG <= 0);
	    for (jG = 0; jG < nG; jG++) {
		 /*
		 *	We construct the G coefficients on-the-fly from the
		 *	Htilde coefficients.
		 *
		 *	Like H, each row of G is offset by 2 from the previous
		 *	one.  As with H, we also allow the coefficents of G to
		 *	wrap.
		 *
		 *	Again as before, the aIn[] values may wrap.
		 */
		iA = Utils.wrapMod(2 * i + jG - wfltr.offG, n);
		if (flip)
		    sum -= wfltr.cHtilde[nG - 1 - jG] * aIn[incA * iA + aInStart];
		else
		    sum += wfltr.cHtilde[nG - 1 - jG] * aIn[incA * iA + aInStart];
		flip = !flip;
	    }
	    aTmp[nDiv2 + i] = sum;
	    }
	}
	else {
		/*
		 *	The inverse transform is a little trickier to do 
		 *      efficiently.
		 *	A single step of the reconstruction is summarized by:
		 *
		 *		aTmp = Htilde^t * aIn[incA * (0..n/2-1)]
		 *				+ Gtilde^t * aIn[incA * 
		 *  (n/2..n-1)]
		 *
		 *	where x^t is the transpose of x.
		 */
	    for (i = 0; i < n; i++)
		aTmp[i] = 0.0;	/* necessary */
	    for (j = 0; j < nDiv2; j++) {
		aInVal = aIn[incA * j + aInStart];
		for (iHtilde = 0; iHtilde < wfltr.nHtilde; iHtilde++) {
		 /*
		  *    Each row of Htilde is offset by 2 from the previous one.
		  */
		    iA = Utils.wrapMod(2 * j + iHtilde - wfltr.offHtilde, n);
		    aTmp[iA] += wfltr.cHtilde[iHtilde] * aInVal;
		}
		/*
		 *	imsGtilde <= 0 <=> initial flip
		 */
		flip = (wfltr.imsGtilde <= 0);
		aInVal = aIn[incA * (j + nDiv2) + aInStart];
		for (iGtilde = 0; iGtilde < nGtilde; iGtilde++) {
	       /*
		*	As with Htilde, we also allow the coefficents of 
		*       Gtilde,
		*	which is the mirror of H, to wrap.
		*
		*	We assume our data is periodic, so we wrap the aIn[] 
		*       values
		*	if necessary.  If we have more samples than we have 
		*       Gtilde
		*	coefficients, we also wrap the Gtilde coefficients.
		*/
		    iA = Utils.wrapMod(2 * j + iGtilde - wfltr.offGtilde, n);
		    if (flip)
			aTmp[iA] -= wfltr.cH[nGtilde - 1 - iGtilde] * aInVal;
		    else
			aTmp[iA] += wfltr.cH[nGtilde - 1 - iGtilde] * aInVal;
		    flip = !flip;
		}
	    }
	}
	for (i = 0; i < n; i++)
	    aXf[incA * i + aXfStart] = aTmp[i];	
	return;
    }
}
