package wavelet;
/**
 *  static arrays of different Wavelet filters

    These Wavelet filters were ported from the package  'wvlt' written 
    in C to Java by Reynald Hoskinson, 1999-2001. 
    
    original disclaimer:

    wvlt: The UBC Imager Wavelet Package -- Release 2.1
    ---------------------------------------------------
    Disclaimer
    ----------
    
    Bob Lewis produced this package and hereby releases it to the public
    domain.  Neither the author nor the University of British Columbia will be
    held responsible for its use, misuse, or abuse or any damages arising from
    same.
    
    Any comments regarding this package may, nevertheless, be sent to
    "bobl@cs.ubc.ca".
    
    Any comments about the port can be sent to: reynald@cs.ubc.ca
 *
 */

public class WaveletFilter {
    
    static final double M_SQRT2 = 1.41421356237309504880168872420969808;
    static final double SQRT15  = 3.87298334620741688517927;
    static final double SQRT3   = 1.73205080756887729352745;
    
    public String name;
    public double[] cH; 
    public int nH;
    public double[] cHtilde;
    public int nHtilde;
    
    /* offsets of H, G, H~, and G~ */			 
    
    public int offH;
    public int offG;
    public int offHtilde; 
    public int offGtilde;

    /* initial mirror signs of G and G~
       (these eliminate the need for filter padding) */
    public int imsG;
    public int imsGtilde;
		
    public WaveletFilter(String iname,
			 double [] icH,
			 int inH,
			 double[] icHtilde,
			 int inHtilde,
			 int ioffH,
			 int ioffG, 
			 int ioffHtilde,
			 int ioffGtilde,
			 int iimsG,
			 int iimsGtilde)
    {
	name = iname;
	cH = icH;
	nH = inH;
	cHtilde = icHtilde;
	nHtilde = inHtilde;
	offH = ioffH;
	offG = ioffG;
	offHtilde = ioffHtilde;
	offGtilde = ioffGtilde;
	imsG = iimsG;
	imsGtilde = iimsGtilde;
	
    }
    
    /**
     * a list of all currently availabe Wavelet filters. 
     */
    public static String[] WaveletFilterList = {
	"AdelsonSimoncelliHingorani",
	"AntoniniBarlaudMatheiuDaubechies_4_4",
	"BattleLemarie",
	"BurtAdelson",
	"Coiflet_2",
	"Coiflet_4",
	"Coiflet_6",
	"Daubechies_4",
	"Daubechies_6",
	"Daubechies_8",
	"Daubechies_10",
	"Daubechies_12",
	"Daubechies_20",
	"Haar",
	"Pseudocoiflet_4_4",
	"Spline_2_2",
	"Spline_2_4",
	"Spline_3_3",
	"Spline_3_5",
	"Spline_3_7"
    };		     		     		   		   		     		   		   		     		  		   		     		     		    		     		  	
    /**
     *	-----------------------------------------------------------------
     *	Adelson-Simoncelli-Hingorani filter
     *
     *	Source: Adelson, Simoncelli, Hingorani, "Orthogonal Pyramid Transforms
     *	  for Image Coding", Visual Communications and Image Processing II,
     *	  Proc. SPIE Vol. 845, pp. 50-58.
     */
    private static double cHAdelsonSimoncelliHingorani[] = {
	M_SQRT2 * 0.01995,
	M_SQRT2 * -0.04271,
	M_SQRT2 * -0.05224,
	M_SQRT2 * 0.29271,
	M_SQRT2 * 0.56458,
	M_SQRT2 * 0.29271,
	M_SQRT2 * -0.05224,
	M_SQRT2 * -0.04271,
	M_SQRT2 * 0.01995
    };
  
    /** 
     * waveletFilter wrapping
     */
  public static WaveletFilter wfltrAdelsonSimoncelliHingorani =
  new WaveletFilter(
		    "AdelsonSimoncelliHingorani",
		    cHAdelsonSimoncelliHingorani,
		    cHAdelsonSimoncelliHingorani.length,
		    cHAdelsonSimoncelliHingorani,
		    cHAdelsonSimoncelliHingorani.length,
		    /* offsets of H, G, Htilde, and Gtilde, respectively */
		    4, 3, 4, 3,
		    /* initial mirror signs for G and Gtilde, respectively */
		    1, 1
		    );
  
  
    /**
     *  -----------------------------------------------------------
     *  Antonini et al.  filters
     *
     *  Source: M. Antonini, M. Barlaud, P. Mathieu, I. Daubechies,
     *	  "Image Coding Using Wavelet Transforms", IEEE Trans. on Image
     *	  Processing, Vol. 1 No. 2, April 1992
     *
     *    (Table II - Spline Variant with Less Dissimilar Lengths)
     *
     *	  Contributed by Brendt Wohlberg (brendt@dip1.ee.uct.ac.za).
     */
  private static double cHAntoniniBarlaudMatheiuDaubechies_4[] = {
    M_SQRT2 *  0.026749,
    M_SQRT2 * -0.016864,
    M_SQRT2 * -0.078223,
    M_SQRT2 *  0.266864,
    M_SQRT2 *  0.602949,
    M_SQRT2 *  0.266864,
    M_SQRT2 * -0.078223,
    M_SQRT2 * -0.016864,
    M_SQRT2 *  0.026749
  };

    /** 
     * waveletFilter wrapping
     */
  private static double cHtildeAntoniniBarlaudMatheiuDaubechies_4[] = {
    M_SQRT2 * -0.045636,
    M_SQRT2 * -0.028772,
    M_SQRT2 *  0.295636,
    M_SQRT2 *  0.557543,
    M_SQRT2 *  0.295636,
    M_SQRT2 * -0.028772,
    M_SQRT2 * -0.045636
  };
  
  public static WaveletFilter wfltrAntoniniBarlaudMatheiuDaubechies_4_4 =
  new WaveletFilter("AntoniniBarlaudMatheiuDaubechies_4_4",
		    cHAntoniniBarlaudMatheiuDaubechies_4,
		    cHAntoniniBarlaudMatheiuDaubechies_4.length,
		    cHtildeAntoniniBarlaudMatheiuDaubechies_4,
		    cHtildeAntoniniBarlaudMatheiuDaubechies_4.length,
		    /* offsets of H, G, Htilde, and Gtilde, respectively */
		    4, 2, 3, 3,
		    /* initial mirror signs for G and Gtilde, respectively */
		    -1, 1
		    );
  
    /**
     *	-----------------------------------------------------------------
     *	Battle-Lemarie filter
     *
     *	Source: Mallat, "A Theory for Multiresolution Signal Decomposition: The
     *	  Wavelet Representation", IEEE PAMI, v. 11, no. 7, 674-693, Table 1
     */
    private static double cHBattleLemarie[] = {
	M_SQRT2 * -0.002,
	M_SQRT2 * -0.003,
	M_SQRT2 *  0.006,
	M_SQRT2 *  0.006,
	M_SQRT2 * -0.013,
	M_SQRT2 * -0.012,
	M_SQRT2 *  0.030,  /*  5 and 6 sign change from Mallat's paper */
	M_SQRT2 *  0.023,
	M_SQRT2 * -0.078,
	M_SQRT2 * -0.035,
	M_SQRT2 *  0.307,
	M_SQRT2 *  0.542,
	M_SQRT2 *  0.307,
	M_SQRT2 * -0.035,
	M_SQRT2 * -0.078,
	M_SQRT2 *  0.023,
	M_SQRT2 *  0.030,
	M_SQRT2 * -0.012,
	M_SQRT2 * -0.013,
	M_SQRT2 *  0.006,
	M_SQRT2 *  0.006,
	M_SQRT2 * -0.003,
	M_SQRT2 * -0.002
    };
    
    public static WaveletFilter wfltrBattleLemarie =
    new WaveletFilter("BattleLemarie",
		      cHBattleLemarie,
		      cHBattleLemarie.length,
		      cHBattleLemarie,
		      cHBattleLemarie.length,
		      /* offsets of H, G, Htilde, and Gtilde, respectively */
		      11, 10, 11, 10,
		      /* initial mirror signs for G and Gtilde, respectively */
		      -1, -1
		      );
    
    /**
     *	-----------------------------------------------------------------
     *	Burt-Adelson filter
     *
     *	Source: TLoW, Table 8.4
     */
    private  static double cHBurtAdelson[] = {
	M_SQRT2 * -1.0 / 20.0,
	M_SQRT2 *  5.0 / 20.0,
	M_SQRT2 * 12.0 / 20.0,
	M_SQRT2 *  5.0 / 20.0,
	M_SQRT2 * -1.0 / 20.0
    };
  
    private  static double cHtildeBurtAdelson[] = {
	M_SQRT2 *  -3.0 / 280.0,
	M_SQRT2 * -15.0 / 280.0,
	M_SQRT2 *  73.0 / 280.0,
	M_SQRT2 * 170.0 / 280.0,
	M_SQRT2 *  73.0 / 280.0,
	M_SQRT2 * -15.0 / 280.0,
	M_SQRT2 *  -3.0 / 280.0
    };
    
    public static WaveletFilter wfltrBurtAdelson =
    new WaveletFilter("BurtAdelson",
		      cHBurtAdelson,
		      cHBurtAdelson.length,
		      cHtildeBurtAdelson,
		      cHtildeBurtAdelson.length,
		      /* offsets of H, G, Htilde, and Gtilde, respectively */
		      2, 2, 3, 1,
		      /* initial mirror signs for G and Gtilde, respectively */
		      -1, 1
		      );
    
    /**
     *	-----------------------------------------------------------------
     *	Coiflet filters
     *
     *	Source: Beylkin, Coifman, and Rokhlin "Fast Wavelet Transforms and
     *	  Numerical Algorithms I", Comm. Pure Appl. Math, v. 44, Appendix A
     */
      
    private static double cHCoiflet_2[] = {
	M_SQRT2 * (SQRT15 - 3) / 32.0,
	M_SQRT2 * (1 - SQRT15) / 32.0,
	M_SQRT2 * (6 - 2 * SQRT15) / 32.0,
	M_SQRT2 * (2 * SQRT15 + 6) / 32.0,
	M_SQRT2 * (SQRT15 + 13) / 32.0,
	M_SQRT2 * (9 - SQRT15) / 32.0
    };
    
    public static WaveletFilter wfltrCoiflet_2 =
    new WaveletFilter("Coiflet_2",
		      cHCoiflet_2,
		      cHCoiflet_2.length,
		      cHCoiflet_2,
		      cHCoiflet_2.length,
		      /* offsets of H, G, Htilde, and Gtilde, respectively */
		      4, 0, 4, 0,
		      /* initial mirror signs for G and Gtilde, respectively */
		      -1, -1);
  
  private static double cHCoiflet_4[] = {
	0.0011945726958388,
	-0.01284557955324,
	0.024804330519353,
	0.050023519962135,
	-0.15535722285996,
	-0.071638282295294,
	0.57046500145033,
	0.75033630585287,
	0.28061165190244,
	-0.0074103835186718,
	-0.014611552521451,
	-0.0013587990591632
    };
    
    public static WaveletFilter wfltrCoiflet_4 = 
    new WaveletFilter("Coiflet_4",
		      cHCoiflet_4,
		      cHCoiflet_4.length,
		      cHCoiflet_4,
		      cHCoiflet_4.length,
		      /* offsets of H, G, Htilde, and Gtilde, respectively */
		      7, 3, 7, 3,
		      /* initial mirror signs for G and Gtilde, respectively */
		      1, 1);
    
  static double cHCoiflet_6[] = {
    -0.0016918510194918,
    -0.00348787621998426,
    0.019191160680044,
    0.021671094636352,
    -0.098507213321468,
    -0.056997424478478,
    0.45678712217269,
    0.78931940900416,
    0.38055713085151,
    -0.070438748794943,
    -0.056514193868065,
    0.036409962612716,
    0.0087601307091635,
    -0.011194759273835,
    -0.0019213354141368,
    0.0020413809772660,
    0.00044583039753204,
    -0.00021625727664696
  };
    
  public static WaveletFilter wfltrCoiflet_6 =
  new WaveletFilter("Coiflet_6",
		    cHCoiflet_6,
		    cHCoiflet_6.length,
		    cHCoiflet_6,
		    cHCoiflet_6.length,
		    /* offsets of H, G, Htilde, and Gtilde, respectively */
		    7, 9, 7, 9,
		    /* initial mirror signs for G and Gtilde, respectively */
		    1, 1
		    );
  
    /**
     *	-----------------------------------------------------------------
     *	Daubechies filters
     *
     *	Source: TLoW, Table 6.1
     */
    
    static double cHDaubechies_4[] = {
	M_SQRT2 * (1 + SQRT3) / 8.0,
	M_SQRT2 * (3 + SQRT3) / 8.0,
	M_SQRT2 * (3 - SQRT3) / 8.0,
	M_SQRT2 * (1 - SQRT3) / 8.0
    };
    
  public static WaveletFilter wfltrDaubechies_4 =
  new WaveletFilter("Daubechies_4",
		   cHDaubechies_4,
		   cHDaubechies_4.length,
		   cHDaubechies_4,
		   cHDaubechies_4.length,
		   /* offsets of H, G, Htilde, and Gtilde, respectively */
		   1, 1, 1, 1,
		   /* initial mirror signs for G and Gtilde, respectively */
		   1, 1
		   );
  
  static double cHDaubechies_6[] = {
      0.332670552950,
      0.806891509311,
      0.459877502118,
      -0.135011020010,
      -0.085441273882,
      0.035226291882
  };
  
  public static WaveletFilter wfltrDaubechies_6 =
  new WaveletFilter ("Daubechies_6",
		     cHDaubechies_6,
		     cHDaubechies_6.length,
		     cHDaubechies_6,
		     cHDaubechies_6.length,
		     /* offsets of H, G, Htilde, and Gtilde, respectively */
		     1, 3, 1, 3,
		     /* initial mirror signs for G and Gtilde, respectively */
		     1, 1);

    static double cHDaubechies_8[] = {
      0.230377813309,
      0.714846570553,
      0.6308807667930,
      -0.027983769417,
      -0.187034811719,
      0.030841381836,
      0.032883011667,
      -0.010597401785
    };
    
  public static WaveletFilter wfltrDaubechies_8 =
  new WaveletFilter("Daubechies_8",
		    cHDaubechies_8,
		    cHDaubechies_8.length,
		    cHDaubechies_8,
		    cHDaubechies_8.length,
		    /* offsets of H, G, Htilde, and Gtilde, respectively */
		    1, 5, 1, 5,
		    /* initial mirror signs for G and Gtilde, respectively */
		    1, 1
		    );

  static double cHDaubechies_10[] = {
    0.1601023979741929,
    0.6038292697971895,
    0.7243085284377726,
    0.1384281459013203,
    -0.2422948870663823,
    -0.0322448695846381,
    0.0775714938400459,
    -0.0062414902127983,
    -0.0125807519990820,
    0.0033357252854738
  };
  
  public static WaveletFilter wfltrDaubechies_10 =
  new WaveletFilter("Daubechies_10",
		    cHDaubechies_10,
		    cHDaubechies_10.length,
		    cHDaubechies_10,
		    cHDaubechies_10.length,
		    /* offsets of H, G, Htilde, and Gtilde, respectively */
		    1, 7, 1, 7,
		    /* initial mirror signs for G and Gtilde, respectively */
		    1, 1
		    );
    
  static double cHDaubechies_12[] = {
    0.1115407433501095,
    0.4946238903984533,
    0.7511339080210959,
    0.3152503517091982,
    -0.2262646939654400,
    -0.1297668675672625,
    0.0975016055873225,
    0.0275228655303053,
    -0.0315820393184862,
    0.0005538422011614,
    0.0047772575119455,
    -0.0010773010853085
  };
  
  public static WaveletFilter wfltrDaubechies_12 =
  new WaveletFilter("Daubechies_12",
		    cHDaubechies_12,
		    cHDaubechies_12.length,
		    cHDaubechies_12,
		    cHDaubechies_12.length,
		    /* offsets of H, G, Htilde, and Gtilde, respectively */
		    1, 9, 1, 9,
		    /* initial mirror signs for G and Gtilde, respectively */
		    1, 1
		    );
  
  static double cHDaubechies_20[] = {
      0.026670057901,
      0.188176800078,
      0.527201188932,
      0.688459039454,
      0.281172343661,
      -0.249846424327,
      -0.195946274377,
      0.127369340336,
      0.093057364604,
      -0.071394147166,
      -0.029457536822,
      0.033212674059,
      0.003606553567,
      -0.010733175483,
      0.001395351747,
      0.001992405295,
      -0.000685856695,
      -0.000116466855,
      0.000093588670,
      -0.000013264203
  };
    
    public static WaveletFilter wfltrDaubechies_20 =
    new WaveletFilter("Daubechies_20",
		      cHDaubechies_20,
		      cHDaubechies_20.length,
		      cHDaubechies_20,
		      cHDaubechies_20.length,
		      /* offsets of H, G, Htilde, and Gtilde, respectively */
		      2, 16, 2, 16,
		      /* initial mirror signs for G and Gtilde, respectively */
		      1, 1
		      );
  
    /**
     *	-----------------------------------------------------------------
     *	Haar filter
     *
     *	Source: TLoW, p. 10 (and lots of other places!)
     */

    static double cHHaar[] = {
	M_SQRT2 * 1.0 / 2.0,
	M_SQRT2 * 1.0 / 2.0
    };
    
  public static WaveletFilter wfltrHaar =
  new WaveletFilter("Haar",
		    cHHaar,
		    cHHaar.length,
		    cHHaar,
		    cHHaar.length,
		    /* offsets of H, G, Htilde, and Gtilde, respectively */
		    0, 0, 0, 0,
		    /* initial mirror signs for G and Gtilde, respectively */
		    1, 1
		    );
    
  /*
   *	-----------------------------------------------------------------
   *  Pseudocoiflet filter
   *  Source: Reissell, "Multiresolution Geometric Algorithms Using Wavelets I:
   *  Representation for Parametric Curves and Surfaces", UBC TR 93-17, p. 33
   */
    
    static double cHPseudocoiflet_4[] = {
	M_SQRT2 *  -1.0 / 512.0,
	0.0,
	M_SQRT2 *  18.0 / 512.0,
	M_SQRT2 * -16.0 / 512.0,
	M_SQRT2 * -63.0 / 512.0,
	M_SQRT2 * 144.0 / 512.0,
	M_SQRT2 * 348.0 / 512.0,
	M_SQRT2 * 144.0 / 512.0,
	M_SQRT2 * -63.0 / 512.0,
	M_SQRT2 * -16.0 / 512.0,
	M_SQRT2 *  18.0 / 512.0,
	0.0,
	M_SQRT2 *  -1.0 / 512.0
    };
  
    static double cHtildePseudocoiflet_4[] = {
	M_SQRT2 *  -1.0 / 32.0,
	0.0,
	M_SQRT2 *   9.0 / 32.0,
	M_SQRT2 *  16.0 / 32.0,
	M_SQRT2 *   9.0 / 32.0,
	0.0,
	M_SQRT2 *  -1.0 / 32.0
    };
  
  public static WaveletFilter wfltrPseudocoiflet_4_4 = 
  new WaveletFilter("Pseudocoiflet_4_4",
		    cHPseudocoiflet_4,
		    cHPseudocoiflet_4.length,
		    cHtildePseudocoiflet_4,
		    cHtildePseudocoiflet_4.length,
		    /* offsets of H, G, Htilde, and Gtilde, respectively */
		    6, 2, 3, 5,
		    /* initial mirror signs for G and Gtilde, respectively */
		    -1, 1
		    );
  
    /**
     *	-----------------------------------------------------------------
     *	Spline filters
     *
     *	Source: TLoW, Table 8.2 (corrected)
     */
    
    static double cHSpline_2[] = {
	M_SQRT2 * -0.125,
	M_SQRT2 *  0.25,
	M_SQRT2 *  0.75,
	M_SQRT2 *  0.25,
	M_SQRT2 * -0.125
    };
    
  static double cHSpline_3[] = {
      M_SQRT2 *  3.0 / 64.0,
      M_SQRT2 * -9.0 / 64.0,
      M_SQRT2 * -7.0 / 64.0,
      M_SQRT2 * 45.0 / 64.0,
      M_SQRT2 * 45.0 / 64.0,
      M_SQRT2 * -7.0 / 64.0,
      M_SQRT2 * -9.0 / 64.0,
      M_SQRT2 *  3.0 / 64.0
  };
  
  static double cHSpline_4[] = {
      M_SQRT2 *   3.0 / 128.0,
      M_SQRT2 *  -6.0 / 128.0,
      M_SQRT2 * -16.0 / 128.0,
      M_SQRT2 *  38.0 / 128.0,
      M_SQRT2 *  90.0 / 128.0,
      M_SQRT2 *  38.0 / 128.0,
      M_SQRT2 * -16.0 / 128.0,
      M_SQRT2 *  -6.0 / 128.0,
      M_SQRT2 *   3.0 / 128.0
  };
  
  static double cHSpline_5[] = {
      M_SQRT2 *  -5.0 / 512.0,
      M_SQRT2 *  15.0 / 512.0,
      M_SQRT2 *  19.0 / 512.0,
      M_SQRT2 * -97.0 / 512.0,
      M_SQRT2 * -26.0 / 512.0,
      M_SQRT2 * 350.0 / 512.0,
      M_SQRT2 * 350.0 / 512.0,
      M_SQRT2 * -26.0 / 512.0,
      M_SQRT2 * -97.0 / 512.0,
      M_SQRT2 *  19.0 / 512.0,
      M_SQRT2 *  15.0 / 512.0,
      M_SQRT2 *  -5.0 / 512.0
  };
  
  static double cHSpline_7[] = {
      M_SQRT2 *   -35.0 / 16384.0,
      M_SQRT2 *  -105.0 / 16384.0,
      M_SQRT2 *  -195.0 / 16384.0,
      M_SQRT2 *   865.0 / 16384.0,
      M_SQRT2 *   363.0 / 16384.0,  /* corrected ("363" was "336") in text) */
      M_SQRT2 * -3489.0 / 16384.0,
      M_SQRT2 *  -307.0 / 16384.0,
      M_SQRT2 * 11025.0 / 16384.0,
      M_SQRT2 * 11025.0 / 16384.0,
      M_SQRT2 *  -307.0 / 16384.0,
      M_SQRT2 * -3489.0 / 16384.0,
      M_SQRT2 *   363.0 / 16384.0,  /* corrected ("363" was "336") in text) */
      M_SQRT2 *   865.0 / 16384.0,
      M_SQRT2 *  -195.0 / 16384.0,
      M_SQRT2 *  -105.0 / 16384.0,
      M_SQRT2 *   -35.0 / 16384.0
  };

    static double cHtildeSpline_2[] = {
	M_SQRT2 * 1.0 / 4.0,
	M_SQRT2 * 2.0 / 4.0,
	M_SQRT2 * 1.0 / 4.0
    };
    static double cHtildeSpline_3[] = {
	M_SQRT2 * 1.0 / 8.0,
	M_SQRT2 * 3.0 / 8.0,
	M_SQRT2 * 3.0 / 8.0,
	M_SQRT2 * 1.0 / 8.0
    };
    
    public static WaveletFilter wfltrSpline_2_2 =
    new WaveletFilter("Spline_2_2",
		      cHSpline_2,
		      cHSpline_2.length,
		      cHtildeSpline_2,
		      cHtildeSpline_2.length,
		      /* offsets of H, G, Htilde, and Gtilde, respectively */
		      2, 0, 1, 1,
		      /* initial mirror signs for G and Gtilde, respectively */
		      -1, 1
		      );
    
  public static WaveletFilter wfltrSpline_2_4 = 
  new WaveletFilter("Spline_2_4",
		    cHSpline_4,
		    cHSpline_4.length,
		    cHtildeSpline_2,
		    cHtildeSpline_2.length,
		    /* offsets of H, G, Htilde, and Gtilde, respectively */
		    4, 0, 1, 3,
		    /* initial mirror signs for G and Gtilde, respectively */
		    -1, 1
		    );
    
  public static WaveletFilter wfltrSpline_3_3 = 
  new WaveletFilter("Spline_3_3",
		    cHSpline_3,
		    cHSpline_3.length,
		    cHtildeSpline_3,
		    cHtildeSpline_3.length,
		    /* offsets of H, G, Htilde, and Gtilde, respectively */
		    3, 1, 1, 3,
		    /* initial mirror signs for G and Gtilde, respectively */
		    1, 1
		    );
  
    public static WaveletFilter wfltrSpline_3_5 = 
    new WaveletFilter("Spline_3_5",
		      cHSpline_5,
		      cHSpline_5.length,
		      cHtildeSpline_3,
		      cHtildeSpline_3.length,
		      /* offsets of H, G, Htilde, and Gtilde, respectively */
		      5, 1, 1, 5,
		      /* initial mirror signs for G and Gtilde, respectively */
		      1, 1
		      );
    
  public static WaveletFilter wfltrSpline_3_7 = 
  new WaveletFilter("Spline_3_7",
		    cHSpline_7,
		    cHSpline_7.length,
		    cHtildeSpline_3,
		    cHtildeSpline_3.length,
		    /* offsets of H, G, Htilde, and Gtilde, respectively */
		    7, 1, 1, 7,
		    /* initial mirror signs for G and Gtilde, respectively */
		    1, 1
		    );


}

/* there is also another function, 
   wfltr_exchange -- exchange the normal and tilde components of a wavelet
     filter
     this might be interesting to add later. I would have to change the
     class objects to be non-static, however.
*/

