package wavelet;

import  cern.colt.Arrays;
import  utilities.*;
import	java.io.File;
import	java.io.FileOutputStream;
import	java.io.IOException;
import	javax.sound.sampled.AudioFormat;
import	javax.sound.sampled.AudioInputStream;
import	javax.sound.sampled.AudioSystem;
import	javax.sound.sampled.DataLine;
import	javax.sound.sampled.SourceDataLine;
import	javax.sound.sampled.Clip;
import	javax.sound.sampled.LineListener;
import	javax.sound.sampled.LineUnavailableException;
import	javax.sound.sampled.LineEvent;
import  java.util.Vector;

import javax.swing.DefaultBoundedRangeModel;
import javax.swing.JProgressBar;

import sound.AudioSample;
import java.io.File;

/**
 * Breaks up a sound sample into small windows. 
 */
public class WindowedSegmentation {

    /**
     * segments records the transitions that 
     * are greater than the threshold. They define the segments. 
     * kept is the framenumber of all segment start/end points, as an Integer. 
     * i.e. there are segments.length -2 actual segments 
     */
    public Vector segments = new Vector();
    
    public double [] avgAmplitude;

    /**
     * windowSize in samples. 
     */
     public static final int windowSize = 1024;
    // public static final int windowSize = 512;
    
    /**
     * number of samples incremented per window
     */ 
    public static final int winIncrement = 3*windowSize/4;
    //   public static final int winIncrement = 2*windowSize/4;
    // public static final int winIncrement = windowSize;
         
    /** 
     *  number of wavelet levels to calculate 
     */
    private static final int wavLevels = 6;
    
    /** 
     *   total number of windows in a sample
     **/
    private int     numWindows; 

    /**
     * Wavelet data for 1 frame
     */
    private Wavelet waveletWindows;

    /**
     *  controls the threshold for segmentation: percent is the 
     *  percentage of total possible segments in this sample
     */
    public double thresholdPercent = 55;  // 0 to 100 (eventually * threshold)

    /**
     *   wavelet filter to use for processing
     */
    private WaveletFilter wavFilter;

    /**
     * records the differences between each window
     */
    public Transitions   transitions;

    /**
     *  dictates how few windows can be in one segment
     */
    private int minWindows;   

    /**
     * records whether this sample has been properly analyzed yet. 
     */
    public boolean occupied = false;
    private static final boolean    DEBUG  = false;

    // matlab stuff
    private static final boolean    MATLAB = false;
    private FileOutputStream mat_outStream;
    private File             mat_outFile;
    private String []        mat_outputString;
    public AudioSample      audioSample;


    /**
     * clears out all the currently calculated segmentation and transition data
     */
    public void blank() {
	waveletWindows = null;

	if (transitions != null) {	    
	    transitions.blank();
	}
	occupied = false;
	segments.clear();
    }

    /**
     * initializes class - used in CapturePlayback
     * @param audioSample audio signal to process
     */
    public void initialize(AudioSample audiosample) {
	audioSample = audiosample;
	occupied = true;	
	audioSample.setWindowSize(windowSize); 
	audioSample.setWindowIncrement(winIncrement); 
	numWindows = (audioSample.getSignalSizeInSamples()/winIncrement);
	//	System.out.println("numwindows: " + numWindows);
    }

    /**
     * constructor given minWindows
     * @param minWindows how few windows can be in 1 segment
     */
    public WindowedSegmentation(int minWindows1) {
	minWindows = minWindows1;
	if (MATLAB) {
	    try {
		mat_outFile   = new File("graphPoints.m");
		mat_outStream = new FileOutputStream(mat_outFile);
		mat_outputString = new String[7];
		
		String tempString = new String("function a = graphPoints();\n\n");
		mat_outStream.write(tempString.getBytes());
		
		// preparing strings. 
		for (int i = 0; i<wavLevels; i++) {
		    mat_outputString[i] = new String("a(");
		    mat_outputString[i] += i+1 + ",:)=[";
		}
	    }
	    catch (Exception ex) { 
		ex.printStackTrace();
		System.out.println("can't open file: graphPoints");
	    }		    
	}
    }

    /**
     * normalize transition.energies over all wavelet levels (6 at present); 
     * must be called AFTER segment    
     */
    public void normalize() {
	int i,j;
	double normal;
	for (i = 0; i < transitions.energies.length; i++ ) {
	    normal = 0;
	    for (j = 0; j < transitions.energies[i].length; j++ ) {
		normal += transitions.energies[i][j];
		
		if (MATLAB) {
		   if (i< transitions.energies.length-1) {
		       mat_outputString[j] += 
			   transitions.energies[i][j] + " ";
		   }
		   else {
		       // System.out.println("last one: " + i);
		       mat_outputString[j] += 
			   transitions.energies[i][j] + "];\n\n";
		       try {
			   mat_outStream.write(mat_outputString[j].getBytes());
			   mat_outputString[j] = null;
		       }
		       catch (Exception ex) { 
			   ex.printStackTrace();
		       }
		   }
		}
	    }
	    for (j=0; j< transitions.energies[i].length; j++) {
		if (normal > 0) {
		    transitions.energies[i][j] /= normal;
		}
		/*
		if (Double.isNaN(norm)) {
		    System.out.println("normal: " + normal);
		    System.out.println("transitions.energies[" + i + "][" + j + "] "
				       + transitions.energies[i][j]);
		}
		*/
		// matlab stuff went here. 
	    }
	}
    }

    /**
     * creates list of transitions by calculating differences between 
     * wavelet coefficient energies. 
     * format: [WinSeg.segments number][start of segment in bytes] 
     *         [WinSeg.segments number][number of bytes in segment] 
     */
    public int [][]  createTransList() {
		
	// last segment not used -- its the end. 
	int [][] transList = new int[segments.size()-1][];
	int frameIncrement = audioSample.getFrameIncrement();

	transList[0] = new int[2];
	transList[0][0] = 0;
	transList[0][1] = 
	    (((Integer)(segments.get(1))).intValue() +2)*frameIncrement;      

	//System.out.println("First Translist: " + transList[0][0] + "," + transList[0][1]);
	
	int j;
	for (j = 1; j < segments.size()-1; j++) {
	    transList[j] = new int[2];
	    transList[j][0] = 
		((((Integer)(segments.get(j))).intValue()+2)*frameIncrement);
	    
	    transList[j][1] = 
		((((Integer)(segments.get(j+1))).intValue()+2)*frameIncrement)-
		((((Integer)(segments.get(j))).intValue()+2)*frameIncrement); 
	}	
	return transList;
    }

    public static void main(String[] args)
    {
	double threshold = 10.0;
        checkArgs(args);        
	try {
	    AudioSample aus = new AudioSample(new File(args[0]));
	    
	    Integer wfl = new Integer(args[1]);
	    
	    System.out.println("successfully loaded " + args[0]);        
	    
	    // construct object
	    WindowedSegmentation winSeg = new WindowedSegmentation(2);
	    byte[] b = new byte[1];
	    winSeg.initialize(aus);
	}
	catch (Exception ex) { 
	    ex.printStackTrace();
	}
	// FIX 
	//winSeg.separate(wfl.intValue(), b, null);
    }


    /**
     * used if I already know the transitions (ie loading presets)     
     * @param waveletFilter wavelet filter to use
     * @param obj transitions object to use
     */
    public void separateWithTransitions(int waveletFilter, 
					Transitions obj)

    {
	wavFilter = Wavelet.chooseFilter(waveletFilter);
	transitions = obj;
	analyzeTransitions();
	// anything else needed? 
    }

    /**
     * main signal separation routine
     * @param waveletFilter wavelet filter to use
     * @param rangeModel for use in the update window
     * @param progressBar for use in the update window
     */
    public void separate(int waveletFilter,DefaultBoundedRangeModel rangeModel,
			 JProgressBar progressBar) {

	int i, j;
	wavFilter = Wavelet.chooseFilter(waveletFilter);
	transitions = new Transitions(numWindows, wavLevels);
	double[] inputSignal = new double[windowSize];
	int place = 0;
	int sizeInBytes = audioSample.getSizeInBytes();
	int winIncrementInBytes = sizeInBytes*winIncrement; 	
	byte [] audioBytes = audioSample.getAudioBytes();

	for (i = 0; i < numWindows; i++) 
        {
	    if (rangeModel != null) {
		rangeModel.setValue ((int)((double)i/(double)numWindows*100) );
		progressBar.setString((int)(progressBar.getPercentComplete()
					    *100) + "%" );
	    }
	    WavRoutines.getDoubleData(audioBytes, place, 
				      place+winIncrementInBytes, 
				      inputSignal, audioSample.format);
	    waveletWindows = new Wavelet(inputSignal, windowSize);
	    
	    // wavelet decomposition step 
	    waveletWindows.forwardTransform(wavLevels, 0, wavFilter);
	    transitions.avgEnergies[i] = waveletWindows.smoothCoefCost(0);
	    transitions.avgAmp[i] = waveletWindows.avgAmplitude();
	    
	    for (j = 0; j < wavLevels; j++)
	    {		
		transitions.energies[i][j] = waveletWindows.detCoefCost(j);
		// System.out.println("transitions.energies: " 
		// + transitions.energies.length + 
		//  " " + transitions.energies[i].length);
		// matlab output!	
	    }
	    place = place + winIncrementInBytes;
	}
	normalize();
	///////////////////////////////////////////////////
	if (MATLAB) {
	    System.out.println(" ----------- separate --------");
	    // convert time to seconds? 
	    mat_outputString[0] = new String("t=[");
	    
	    double windowIter=0;
	    for (i = 0; i < numWindows-1; i++) {
		mat_outputString[0] += windowIter + ",";
		windowIter += winIncrement;
	    }
	    mat_outputString[0] += windowIter + "];" + "\n";
	    
	    try {
		String plots = new String("xlabel(\'time (seconds)\')\n");
		mat_outStream.write(plots.getBytes());
		plots = "ylabel(\'wavelet energy\')\n";
		mat_outStream.write(plots.getBytes());

		/*
		mat_outStream.write(mat_outputString[0].getBytes());
		plots = "hold\nplot(t,a(1,:),'y')\nfigure\n";
		mat_outStream.write(plots.getBytes());
		plots = "plot(t,a(2,:),'m')\nfigure\n";
		mat_outStream.write(plots.getBytes());
		plots = "plot(t,a(3,:),'c')\nfigure\n";
		mat_outStream.write(plots.getBytes());
		plots = "plot(t,a(4,:),'r')\nfigure\n";
		mat_outStream.write(plots.getBytes());
		plots = "plot(t,a(5,:),'g')\nfigure\n";
		mat_outStream.write(plots.getBytes());
		plots = "plot(t,a(6,:),'b')\nfigure\n";
		mat_outStream.write(plots.getBytes());
		*/
		mat_outStream.write(mat_outputString[0].getBytes());
		plots = "hold\nplot(t,a(1,:),'y')\n";
		mat_outStream.write(plots.getBytes());
		plots = "plot(t,a(2,:),'m')\n";
		mat_outStream.write(plots.getBytes());
		plots = "plot(t,a(3,:),'c')\n";
		mat_outStream.write(plots.getBytes());
		plots = "plot(t,a(4,:),'r')\n";
		mat_outStream.write(plots.getBytes());
		plots = "plot(t,a(5,:),'g')\n";
		mat_outStream.write(plots.getBytes());
		plots = "plot(t,a(6,:),'b')\n";
		mat_outStream.write(plots.getBytes());

	    }	
	    catch (Exception ex) { 
		ex.printStackTrace();
	    }
	}
	//////////////////////////////////////
	fourFrameEuclideanDistance();

	if (MATLAB) {	 
	    try {
		String plots = new String("tr=[");
		for (int v = 0; v < numWindows-1; v++ ) {
		    plots += transitions.trans[v] + " ";
		}
		plots += "0";
		plots += "];\n";
		mat_outStream.write(plots.getBytes());
				
		plots = "hold off\nplot(t,tr,'k')\n";
		mat_outStream.write(plots.getBytes());
		mat_outStream.close();
	    }
	    catch (Exception ex) { 
		ex.printStackTrace();
	    }
	}	    
	analyzeTransitions();
    }

    public void calcAmplitudes() {
	int i,j;
	for (i = 0; i< segments.size()-2; i++) {
	    for (j = ((Integer)segments.get(i)).intValue(); 
		 j < ((Integer)segments.get(i+1)).intValue(); j++) {
		avgAmplitude[i] += transitions.getAvgAmp(j);
	    }
	    avgAmplitude[i] /= j;
	    if (DEBUG) {
		System.out.println("Avg amplitude for segment " + i 
				   + ": " + avgAmplitude[i]);
	    }
	}
    }

    /**
     * calculate which of the transitions between windows can be considered 
     * segmentation points. 
     */
    public void analyzeTransitions() 
    {
	int i;
	int separations=1;
	// get rid of old transitions, if any
	segments.clear();
	segments.add(new Integer(0));
	
	// ---? avgAmplitude.add(new Integer(0));
	double threshlimit = transitions.max - transitions.min;
	//System.out.println("Max: " + transitions.max  + 
	//		   " Min: " + transitions.min);
	double threshold = 
	    Math.pow((thresholdPercent/100.0F),2.9)*threshlimit 
	    + transitions.min;
	
	if (DEBUG) {
	System.out.println("Using thresh: " + threshold);
	}
	for (i = 2; i < transitions.length(); i++) {
	    if ((transitions.trans[i] < threshold) 
		&& transitions.trans[i] > 0) {
		if ((i - ((Integer)segments.get(separations-1)).intValue())
		    >= minWindows) {
		    
		    separations++;
		    segments.add(new Integer(i));
		    // System.out.println("separation at: " + i + ", the " 
		    //		   + winIncrement*i + " sample");
		}
	    }
	}
	// add the last window!
	separations++;
	segments.add(new Integer(transitions.length()-1));
	avgAmplitude = new double[separations];

	if (DEBUG) {
	    System.out.println("There were " + separations + 
			       "separations");
	}
	if (MATLAB) {
	    try {
		String plots = new String("b = [0 " + audioSample.getSignalSizeInSamples() + "]\n");
		mat_outStream.write(plots.getBytes());
		plots = "c = [" + threshold + " " + threshold + "];\n";
		mat_outStream.write(plots.getBytes());
		plots = "plot(b,c,'k')\nhold off\n";	   
		mat_outStream.write(plots.getBytes());
		mat_outStream.close();
	    }
	    catch (Exception ex) { 
		ex.printStackTrace();
	    }
	}
    }

    /**
     * routine for determining the difference between windows
     * dependant on their wavelet energies. 
     */
    public void fourFrameEuclideanDistance() {
	int i, j, k, a;
	int m = 6;
	int maxpt = 2;
	double min2 = 100;
	double max2 = 0;
	/*
	for (i= 0; i < transitions.energies.length; i++) {
	    System.out.println("energ" + i + ": " + Arrays.toString(transitions.energies[i]));
	}
	*/

	String plots;
	if (MATLAB) {	 
		plots = new String("hold off\nfigure\ntitle('Untrimmed transitions');\n xlabel(\'time (seconds)\')\n");		
		plots += "ylabel(\'difference energy\')\n";
		plots += "unTrimmed=[0 0 ";
	}

	for (a = 0; a < numWindows - 3; a++) 
	{	    	    
	    // System.out.println("calculating distance of window: " + a);
	    for (i=a; i< a+2; i++) 
	    { 	     
		for (j=a+2; j< a+4; j++) 
		{
		
		    for(k = 0; k < m; k++) 
		    {
			transitions.trans[a] += 
			    Math.pow(transitions.energies[i][k] - 
				     transitions.energies[j][k], 2);
		    }
		}
	    }

	    if ((transitions.trans[a] < min2) 
		&& (transitions.trans[a] > 0.0)) {
		transitions.min = min2;
		min2 = transitions.trans[a];

	    }
	    if (transitions.trans[a] > max2) {
		transitions.max = max2;
		max2 = transitions.trans[a];
	    }			

	    if (MATLAB) {
		plots += transitions.trans[a] + " ";
	    }

	    if ((a > 0) && (transitions.trans[a-1] > 0.0)) {
		// want only the maximum of immediate neighbors. 
		if (transitions.trans[a-1] < transitions.trans[a]) 
		    {
			transitions.trans[a] = 0;
		    }
		else {
		    transitions.trans[a-1] = 0;		    
		}	
	    // System.out.println("transition[a]: " + transitions[a]);
	    }	
	}

	if (MATLAB) {
	    plots += "0";
	    plots += "];\n";
	    try {
		mat_outStream.write(plots.getBytes());
		plots = "plot(t,unTrimmed,'k')\nfigure\n";
		mat_outStream.write(plots.getBytes());
	    }			   
	    catch (Exception ex) { 
		ex.printStackTrace();
	    }
	}	    
	
	if (DEBUG) {
	    System.out.println("max transition: " + transitions.max);
	    System.out.println("min transition: " + transitions.min);
	}
    }

    /*
      how about a function that given four windows, computes ffeud
      how to specify windows? ints?

      a,b refer each refer to two adjacent windows. 
      they are indexes into the transitions.energies[][] array. 
      
    */
    
    /**
     * routine for determining the difference between two specfic windows
     * dependant on their wavelet energies. 
     * @param a window number a
     * @param b window number b
     * @return difference
     */
    public double fourFrameEuclideanDistance(int a, int b) {
	int i, j, k;
	double result = 0;

	//System.out.println("calculating distance of window: " + a + " " + b);
	for (i=a; i< a+2; i++) 
	    { 	     
		for (j=b; j< b+2; j++) 
		{
		
		    for(k = 0; k < wavLevels; k++) 
		    {
			result += Math.pow(transitions.energies[i][k] - 
					   transitions.energies[j][k], 2);
		    }
		}
	    }
	/*
	double semi;
	if (Double.isNaN(result)) {
	    System.out.println("got nan!");
	    for (i=a; i< a+2; i++) 
		{ 	     
		    for (j=b; j< b+2; j++) 
			{
			    
			    for(k = 0; k < wavLevels; k++) 
				{
				    semi = Math.pow(transitions.energies[i][k] -
						    transitions.energies[j][k], 2);
				    if (Double.isNaN(semi)) {
					System.out.println(transitions.energies[i][k] 
							   +  " " + transitions.energies[j][k]);
				    }
				    
				}
			}	       
		}
	    }
	*/
	result /= (double)4.0;  
	return result;
    }
    
    /**
     * utility for checking arguments. 
     */
    private static void checkArgs(String[] args) 
    {
	if (args.length != 2) {
	    System.out.println("Window: usage:");
	    System.out.println("\tjava Window <soundfile> <waveletFilter>");
	    System.exit(1);
	}
    }

    /**
     * returns an array of how compatible each
     * segment is to the one denoted by inx using start of each segment
     * @param inx segment number
     * @return array of differences of all other windows to input one. 
     */
    public double[] doCompatAtStart(int inx) {
	// System.out.println("doCompatAtStart");
	int i; 
	/*
	  start of this segment and the start of each of the rest. 
	*/
	int numSegs = segments.size()-1;
	double [] compat = new double[numSegs];
	int startOfThisSegment = 0;

	// double min = 10.0;
	// double max = -10.0;
	if (inx > 0) {
	    startOfThisSegment = ((Integer)segments.get(inx-1)).intValue();
	}
	for (i=0; i < numSegs; i++) {	 
	    /*
	     System.out.println("comparing " + startOfThisSegment + " and " + 
				((Integer)segments.get(i)).intValue());
	    if (startOfThisSegment == ((Integer)segments.get(i)).intValue()) {
		System.out.println("compat start: " + startOfThisSegment
				   + ": " + compat[i]);
	    }
	    */
	     compat[i] = fourFrameEuclideanDistance(startOfThisSegment,
				   ((Integer)segments.get(i)).intValue());

	}
	return compat;
    }    

    /**
     * returns an array of how compatible each
     * segment is to the one denoted by inx using the middle of each segment. 
     * @param inx reference frame number of end of segment
     * @return array of differences of all other windows to input one. 
     */
    public double[] doCompatAtMiddle(int inx) {
	// 	System.out.println("doCompatAtMiddle");
	int i; 
	/*
	  middle of this frame and the middle of each of the rest. 
	*/
	int numSegs = segments.size()-1;
	double []compat = new double[numSegs];
	
	int inxMiddle;
	int iMiddle;
	int ini;
	// double min = 10.0;
	// double max = -10.0;

	if( inx == 0) {
	    inxMiddle = (((Integer)segments.get(inx)).intValue())/2;
	    compat[0] = 0;
	}
	else  {
	    ini = ((Integer)segments.get(inx-1)).intValue(); 
	    inxMiddle = ((((Integer)segments.get(inx)).intValue()) - ini)/2;
	    inxMiddle += ini;
	    iMiddle = (((Integer)segments.get(0)).intValue())/2;

	    // System.out.println("comparing " + inxMiddle + " and " + 
	    //		       iMiddle);
	    compat[0] = 
		fourFrameEuclideanDistance(inxMiddle, iMiddle);
	}
	// System.out.println("middle is : " + inxMiddle);
	for (i=1; i < numSegs; i++) {	 
	    ini = ((Integer)segments.get(i-1)).intValue();
	    iMiddle = 
		((((Integer)segments.get(i)).intValue()) - ini)/2;
	    iMiddle += ini;

	    //	    System.out.println("comparing " + inxMiddle + " and " + 
	    //	       iMiddle);

	    compat[i] = 
		fourFrameEuclideanDistance(inxMiddle, iMiddle);
	    /*
	    if (i == inx) {
		System.out.println("compat middle: " + i + ": " + 
				   compat[i]);
	    }
	    */
	}
	// System.out.println("[middle] min compat: " + min + " max: " + max);
	// deal with wrap case?
	return compat;
    }    

    /**
     * returns an array of how compatible the end of each
     * segment is to the end of the one denoted by inx
     * @param inx reference frame number of end of segment
     * @return array of differences of all other windows to input one. 
     */
    public double[] doCompatAtEnd(int inx) {
	int i; 
	/*
	  end of this window and the end of each of the rest. 
	*/
	int numSegs = segments.size()-1;
	double [] compat = new double[numSegs];
	int endOfInx;
	// double max = -5.0; double min = 5.0;
	// boundary case. 
	if (inx < numSegs) {	    
	    endOfInx = ((Integer)segments.get(inx+1)).intValue()-2;
	}
	else {
	    endOfInx = numWindows - 2;
	}

	int otherIndex;
	for (i=0; i < numSegs-1; i++) {	 
	    otherIndex = ((Integer)segments.get(i+1)).intValue()-2;
	    
	    // System.out.println("comparing windows: " + endOfInx + " and " + 
	    // 		       otherIndex);	    
	    compat[i] = fourFrameEuclideanDistance(endOfInx, otherIndex);
	    /*
	    if (i == inx) {
		System.out.println("compat end: " + i + ": " + 
				   compat[i]);
	    }
	    */
	    /*
	    if (compat[i] < min)
		min = compat[i];
	    else if (compat[i] > max) {
		max = compat[i];
	    }
	    */
	}
	// System.out.println("[end] min compat: " + min + " max: " + max);
	// use numWindows? -- hopefully that is the true boundary -- 
	// check this. 
	compat[numSegs-1] = fourFrameEuclideanDistance(endOfInx, numWindows-2);
	// deal with wrap case?
	return compat;
    }    
    

    /**
     * returns an array of how compatible the start of each
     * segment is to the end of the one denoted by inx
     * @param inx reference frame number of end of segment
     * @return array of differences of all other windows to input one. 
     */
    public double[] doCompatAtJoints(int inx) {
	int i; 
	/*
	  end of this window and the start of each of the rest. 
	*/
	int numSegs = segments.size()-1;
	double [] compat = new double[numSegs];
	int endOfInx;

	// double max = -5.0; double min = 5.0;
	// boundary case. 
	if (inx < numSegs) {	    
	    endOfInx = ((Integer)segments.get(inx+1)).intValue()-2;
	}
	else {
	    endOfInx = numWindows - 2;
	}
	int startOfIndex;
	for (i=0; i < numSegs; i++) {	 		
	    startOfIndex = ((Integer)segments.get(i)).intValue();
	    //    System.out.println("comparing windows: " + endOfInx + 
	    //		     " and " +   startOfIndex);
	    compat[i] = fourFrameEuclideanDistance(endOfInx, startOfIndex);

	    /*
	      if (compat[i] < min)
	      min = compat[i];
	      else if (compat[i] > max) {
	      max = compat[i];
	      }
	    */
	}
	// System.out.println("[end] min compat: " + min + " max: " + max);
	// deal with wrap case?
	return compat;
    }    
}

